//+------------------------------------------------------------------+
//|                                               .mq5 |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#include <ChartObjects\ChartObjectsTxtControls.mqh>
#include <Trade\Trade.mqh>
//---
#define MA_MAGIC 160909
input double Lot = 0;
input double StopPercent = 2;
input double TakePercent = 1.85;
input bool   ShowInputs = true;
double StopLevel, TakeLevel, Price;
//---
int   sy=0;
int hATR;
int lotDigits=2;
CTrade ExtTrade;
double g_lot, g_stopPercent, g_takePercent;

int OnInit(void)
{
   hATR = iATR(NULL,0,10);
   if(hATR==INVALID_HANDLE)
   {
      printf("Error creating iATR indicator");
      return -1;
   }
   ExtTrade.SetExpertMagicNumber(MA_MAGIC);
   ExtTrade.SetMarginMode();
   lotDigits = (int)NormalizeDouble(-MathLog(SymbolInfoDouble(Symbol(),SYMBOL_VOLUME_MIN))/MathLog(10), 0);
   g_lot = Lot;
   g_stopPercent = StopPercent;
   g_takePercent = TakePercent;
//--- call init function
   if(ExtScript.Init()==0)
   {
      EventSetMillisecondTimer(100);
      return INIT_SUCCEEDED;
   }
//---
   return INIT_FAILED;
}
void OnDeinit(const int reason)
{
   ExtScript.Deinit();
}
void OnTick(void)
{
}
void OnTimer(void)
{
   ExtScript.Processing();
}
//+------------------------------------------------------------------+
class CPositionConsole
{
protected:
   //--- chart objects
   CChartObjectButton btnSell;
   CChartObjectButton btnBuy;
   CChartObjectButton btnApplay;
   CChartObjectEdit   inpVolume;
   CChartObjectEdit   inpSlPercent, inpTpPercent;
   CChartObjectLabel  lblVol, lblSL, lblTP;

public:
                     CPositionConsole(void);
                    ~CPositionConsole(void);
   //---
   bool              Init(void);
   void              Deinit(void);
   bool              Processing(void);
   void              showBtnApplay(void);
   void              HideBtnApplay(void);
   void              showLevels(int op);
   void              HideLevels(void);

private:
};
//---
CPositionConsole ExtScript;
//+------------------------------------------------------------------+
//| Constructor                                                      |
//+------------------------------------------------------------------+
CPositionConsole::CPositionConsole(void)
{
}
//+------------------------------------------------------------------+
//| Destructor                                                       |
//+------------------------------------------------------------------+
CPositionConsole::~CPositionConsole(void)
{
}
//+------------------------------------------------------------------+
//| Method Init.                                                     |
//+------------------------------------------------------------------+
bool CPositionConsole::Init(void)
  {
   color color_label;
   color color_info;
//--- tuning colors
   color_info =(color)(ChartGetInteger(0,CHART_COLOR_BACKGROUND)^0xFFFFFF);
   color_label=(color)(color_info^0x202020);
//---
   sy=0;
   if(ChartGetInteger(0,CHART_SHOW_OHLC))
      sy+=30;
//--- creation Buttons
   btnSell.Create(0,"ButtonSell",0,15,sy,80,50);
   btnSell.Description("SELL");
   btnSell.Color(Red);
   btnSell.BackColor(Black);
   btnSell.FontSize(16);
//---
   btnBuy.Create(0,"ButtonBuy",0,110,sy,80,50);
   btnBuy.Description("BAY");
   btnBuy.Color(Green);
   btnBuy.BackColor(Black);
   btnBuy.FontSize(16);
//--- redraw chart
   ChartRedraw();
//---
   return(0);
}

void CPositionConsole::showBtnApplay()
{
//---
   btnApplay.Create(0,"ButtonApplay",0,220,sy,200,50);
   btnApplay.Description("OPEN");
   btnApplay.Color(DodgerBlue);
   btnApplay.BackColor(Black);
   btnApplay.FontSize(16);
   btnApplay.Corner(CORNER_RIGHT_UPPER);
//---
   if(ShowInputs)
   {
//lblVol, lblSL, lblTP
   lblVol.Create(0, "VOL.", 0, 138, sy+121);
   lblVol.Color(clrSilver);
   lblVol.Corner(CORNER_RIGHT_UPPER);
   inpVolume.Create(0,"EditVolume", 0, 60, sy+121, 39, 20);
   inpVolume.Description(DoubleToString(Lot, lotDigits));
   inpVolume.Color(clrWhiteSmoke);
   inpVolume.BackColor();
   inpVolume.FontSize(12);
   inpVolume.Corner(CORNER_RIGHT_UPPER);
//---
   lblSL.Create(0, "SL %", 0, 140, sy+60);
   lblSL.Color(clrRed);
   lblSL.Corner(CORNER_RIGHT_UPPER);
   inpSlPercent.Create(0,"EditSlPercent", 0, 60, sy+60, 40, 20);
   inpSlPercent.Description(DoubleToString(StopPercent,2));
   inpSlPercent.Color(clrRed);
   inpSlPercent.BackColor();
   inpSlPercent.FontSize(12);
   inpSlPercent.Corner(CORNER_RIGHT_UPPER);

//---
   lblTP.Create(0, "TP %", 0, 140, sy+90);
   lblTP.Color(clrGreen);
   lblTP.Corner(CORNER_RIGHT_UPPER);
   inpTpPercent.Create(0,"EditTpPercent", 0, 60, sy+90, 40, 20);
   inpTpPercent.Description(DoubleToString(TakePercent,2));
   inpTpPercent.Color(clrGreen);
   inpTpPercent.BackColor();
   inpTpPercent.FontSize(12);
   inpTpPercent.Corner(CORNER_RIGHT_UPPER);
   }
//---
   ChartRedraw();
   
   PlaySound("tick.wav");
}

void CPositionConsole::showLevels(int op)
{
   double atr[1];
   if(CopyBuffer(hATR,0,0,1,atr)!=1)
   {
      Print("CopyBuffer from hATR failed, no data");
      return;
   }
   double SLShift = atr[0]/MathSqrt(MathSqrt(MathSqrt(Period())))*2;
   double TPShift = SLShift/1*g_takePercent;
   

   MqlTick last;
   SymbolInfoTick(NULL, last);
   
   //double stop, take;
   if(op == ORDER_TYPE_BUY)
   {
      TakeLevel = last.ask+TPShift;
      StopLevel = last.ask-SLShift;
   }
   else
   {
      TakeLevel = last.bid-TPShift;
      StopLevel = last.bid+SLShift;
   }
   showStopLine(StopLevel);
   showTakeLine(TakeLevel);
//+------------------------------------------------------------------------+
}

void CPositionConsole::HideBtnApplay()
{
   btnApplay.Delete();
   if(ShowInputs)
   {
      inpVolume.Delete();
      inpSlPercent.Delete();
      inpTpPercent.Delete();
      lblVol.Delete();
      lblSL.Delete();
      lblTP.Delete();
   }
}

void CPositionConsole::HideLevels()
{
   ObjectDelete(0, "%tp");
   ObjectDelete(0, "%sl");
}
//+------------------------------------------------------------------+
//| Method Deinit.                                                   |
//+------------------------------------------------------------------+
void CPositionConsole::Deinit(void)
{
   HideLevels();
}

enum eOpenState
{
   OpenState_None,
   OpenState_Buy,
   OpenState_Sell,
   OpenState_Applay
}
openState = OpenState_None;
//+------------------------------------------------------------------+
//| Method Processing.                                               |
//+------------------------------------------------------------------+
bool CPositionConsole::Processing(void)
{
   double lot, stopPercent, takePercent, stopLevel, takeLevel, price;
   MqlTick last;
   double SLShift, TPShift;
   ENUM_ORDER_TYPE op = 0;
   
   // /
   if(btnSell.State() && btnBuy.State())
   {
      switch(openState)
      {
         case OpenState_Buy:
            openState = OpenState_None;
            btnBuy.State(false);
            HideBtnApplay();
            HideLevels();
            break;
         case OpenState_Sell:
            openState = OpenState_None;
            btnSell.State(false);
            HideBtnApplay();
            HideLevels();
            break;
      }
   }
//---
   switch(openState)
   {
      case OpenState_None:
         if(btnSell.State())
         {
            showBtnApplay();
            showLevels(ORDER_TYPE_SELL);
            openState = OpenState_Sell;
         }
         if(btnBuy.State())
         {
            showBtnApplay();
            showLevels(ORDER_TYPE_BUY);
            openState = OpenState_Buy;
         }
      break;
      case OpenState_Buy:
      case OpenState_Sell:
         if(!btnSell.State() && !btnBuy.State())
         {
            HideBtnApplay();
            HideLevels();
            openState = OpenState_None;
            break;
         }
         
         // , 
         takeLevel = ObjectGetDouble(0,"%tp",OBJPROP_PRICE);
         stopLevel = ObjectGetDouble(0,"%sl",OBJPROP_PRICE);
         //lot = StringToDouble(inpVolume.GetString(OBJPROP_TEXT));
         if(ShowInputs)
         {
            stopPercent = StringToDouble(inpSlPercent.GetString(OBJPROP_TEXT));
            takePercent = StringToDouble(inpTpPercent.GetString(OBJPROP_TEXT));
         }
         SymbolInfoTick(NULL, last);
         
         //double stop, take;
         if(openState == OpenState_Buy)
         {
            price = last.ask;
            op = ORDER_TYPE_BUY;
         }
         else
         {
            price = last.bid;
            op = ORDER_TYPE_SELL;
         }
         
         if(stopLevel != StopLevel || price != Price || (ShowInputs && (stopPercent != g_stopPercent || takePercent != g_takePercent)))
         {
            if(ShowInputs)
            {
               g_stopPercent = stopPercent;
               g_takePercent = takePercent;
            }
            SLShift = price - stopLevel;
            TPShift = SLShift/1*g_takePercent;
            
            //double stop, take;
            TakeLevel = (openState == OpenState_Buy ? last.ask : last.bid) + TPShift;
            moveTakeLine(TakeLevel);
            
            lot = GetLotSize(op, price, SLShift);
            if(lot != g_lot)
            {
               g_lot = lot;
               if(ShowInputs)  inpVolume.Description(DoubleToString(g_lot, lotDigits));
            }
            
            StopLevel = stopLevel;
            Price = price;
         }

         if(btnApplay.State())
         {
            btnSell.State(false);
            btnBuy.State(false);
            btnApplay.State(false);
            HideBtnApplay();
            HideLevels();
            openState = OpenState_None;
	    ChartRedraw();
            //open position
            if(TerminalInfoInteger(TERMINAL_TRADE_ALLOWED))
               ExtTrade.PositionOpen(_Symbol, op, g_lot, Price, StopLevel, TakeLevel);
         }
      break;
   }
   
//--- redraw chart
   ChartRedraw();
   return true;
}
//+------------------------------------------------------------------+

//+
void showStopLine(double price)
{
   showLine("%sl", price, clrRed, true);
}
//+
void showTakeLine(double price)
{
   showLine("%tp", price, DodgerBlue, false);
}
//+
void showLine(string name, double price, color clr, bool selected)
{
   ObjectCreate(0, name,OBJ_HLINE,0,0,price);
   ObjectSetInteger(0, name,OBJPROP_COLOR,clr);
   ObjectSetInteger(0, name,OBJPROP_STYLE,STYLE_SOLID);
   ObjectSetInteger(0, name,OBJPROP_WIDTH, 2);
   ObjectSetInteger(0, name,OBJPROP_SELECTABLE, 1);
   ObjectSetInteger(0, name,OBJPROP_SELECTED, selected);
}
//+
void moveStopLine(double price)
{
   moveLine("%sl", price);
}
//+
void moveTakeLine(double price)
{
   moveLine("%tp", price);
}
//+
void moveLine(string name, double price)
{
   ObjectSetDouble(0, name, OBJPROP_PRICE, price);
}
//+
double GetLotSize(ENUM_ORDER_TYPE op, double price, double stop)
{
   double maxlot = SymbolInfoDouble(Symbol(),SYMBOL_VOLUME_MAX);
   double minlot = SymbolInfoDouble(Symbol(),SYMBOL_VOLUME_MIN);
   double margin;
   bool res = OrderCalcMargin(op, Symbol(), minlot, price, margin);
   double maxlot_margin = AccountInfoDouble(ACCOUNT_MARGIN_FREE)/margin;
   double point = SymbolInfoDouble(Symbol(),SYMBOL_POINT);
   double sum=AccountInfoDouble(ACCOUNT_MARGIN_FREE)/100*g_stopPercent;
   double pipvalue = SymbolInfoDouble(Symbol(),SYMBOL_TRADE_TICK_VALUE)/(SymbolInfoDouble(Symbol(),SYMBOL_TRADE_TICK_SIZE)/point);
   double lot=sum/(fabs(stop)/point*pipvalue);

   lot = NormalizeDouble(fmin(fmax(lot, minlot), fmin(maxlot, maxlot_margin)), lotDigits);
   return lot;
}
